@@ -259,20 +259,20 @@
 *     All Rights Reserved.
 
 *  Licence:
-*     This program is free software; you can redistribute it and/or
-*     modify it under the terms of the GNU General Public Licence as
-*     published by the Free Software Foundation; either version 2 of
-*     the Licence, or (at your option) any later version.
+*     This program is free software: you can redistribute it and/or
+*     modify it under the terms of the GNU Lesser General Public
+*     License as published by the Free Software Foundation, either
+*     version 3 of the License, or (at your option) any later
+*     version.
 *
-*     This program is distributed in the hope that it will be
-*     useful,but WITHOUT ANY WARRANTY; without even the implied
-*     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-*     PURPOSE. See the GNU General Public Licence for more details.
+*     This program is distributed in the hope that it will be useful,
+*     but WITHOUT ANY WARRANTY; without even the implied warranty of
+*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*     GNU Lesser General Public License for more details.
 *
-*     You should have received a copy of the GNU General Public Licence
-*     along with this program; if not, write to the Free Software
-*     Foundation, Inc., 51 Franklin Street,Fifth Floor, Boston, MA
-*     02110-1301, USA
+*     You should have received a copy of the GNU Lesser General
+*     License along with this program.  If not, see
+*     <http://www.gnu.org/licenses/>.
 
 *  Authors:
 *     RFWS: R.F. Warren-Smith (Starlink)
@@ -756,6 +756,95 @@ void astClear##attribute##_( Ast##class *this, int *status ) { \
 }
 #endif
 
+#if defined(astCLASS)            /* Protected */
+/*
+*+
+*  Name:
+*     astMAKE_CLEAR1
+
+*  Purpose:
+*     Implement a method to clear an attribute value for a class, reporting
+*     an error if the object has more than one reference.
+
+*  Type:
+*     Protected macro.
+
+*  Synopsis:
+*     #include "object.h"
+*     astMAKE_CLEAR1(class,attribute,component,assign)
+
+*  Class Membership:
+*     Defined by the Object class.
+
+*  Description:
+*     This macro expands to an implementation of a private member function of
+*     the form:
+*
+*        static void Clear<Attribute>( Ast<Class> *this )
+*
+*     and an external interface function of the form:
+*
+*        void astClear<Attribute>_( Ast<Class> *this )
+*
+*     which implement a method for clearing a specified attribute value for
+*     a class. An error is reported if the object has a reference count that
+*     is greater than one.
+
+*  Parameters:
+*     class
+*        The name (not the type) of the class to which the attribute belongs.
+*     attribute
+*        The name of the attribute to be cleared, as it appears in the function
+*        name (e.g. Label in "astClearLabel").
+*     component
+*        The name of the class structure component that holds the attribute
+*        value.
+*     assign
+*        An expression that evaluates to the value to assign to the component
+*        to clear its value.
+
+*  Notes:
+*     -  To avoid problems with some compilers, you should not leave any white
+*     space around the macro arguments.
+*-
+*/
+
+/* Define the macro. */
+#define astMAKE_CLEAR1(class,attribute,component,assign) \
+\
+/* Private member function. */ \
+/* ------------------------ */ \
+static void Clear##attribute( Ast##class *this, int *status ) { \
+\
+/* Check the inherited error status. */ \
+   if ( !astOK ) return; \
+\
+/* Report an error if the object has been cloned (i.e. has a reference \
+   count that is greater than one). */ \
+   if( astGetRefCount( this ) > 1 ) { \
+      astError( AST__IMMUT, "astClear(%s): The " #attribute "attribute of " \
+                "the supplied %s cannot be cleared because the %s has " \
+                "been cloned (programming error).", status, \
+                astGetClass(this), astGetClass(this), astGetClass(this) ); \
+\
+/* Otherwise, assign the "clear" value in the structure component. */ \
+   } else { \
+      this->component = (assign); \
+   } \
+} \
+\
+/* External interface. */ \
+/* ------------------- */ \
+void astClear##attribute##_( Ast##class *this, int *status ) { \
+\
+/* Check the inherited error status. */ \
+   if ( !astOK ) return; \
+\
+/* Invoke the required method via the virtual function table. */ \
+   (**astMEMBER(this,class,Clear##attribute))( this, status ); \
+}
+#endif
+
 #if defined(astCLASS)            /* Protected */
 /*
 *+
@@ -1006,6 +1095,94 @@ void astSet##attribute##_( Ast##class *this, type value, int *status ) { \
 }
 #endif
 
+#if defined(astCLASS)            /* Protected */
+/*
+*+
+*  Name:
+*     astMAKE_SET1
+
+*  Purpose:
+*     Implement a method to set an attribute value for a class, reporting
+*     an error if the object has more than one reference.
+
+*  Type:
+*     Protected macro.
+
+*  Synopsis:
+*     #include "object.h"
+*     astMAKE_SET1(class,attribute,type,component,assign)
+
+*  Class Membership:
+*     Defined by the Object class.
+
+*  Description:
+*     This macro expands to an implementation of a private member function of
+*     the form:
+*
+*        static void Set<Attribute>( Ast<Class> *this, <Type> value )
+*
+*     and an external interface function of the form:
+*
+*        void astSet<Attribute>_( Ast<Class> *this, <Type> value )
+*
+*     which implement a method for setting a specified attribute value for a
+*     class. An error is reported if the object has a reference count that
+*     is greater than one.
+
+*  Parameters:
+*      class
+*         The name (not the type) of the class to which the attribute belongs.
+*      attribute
+*         The name of the attribute to be set, as it appears in the function
+*         name (e.g. Label in "astSetLabel").
+*      type
+*         The C type of the attribute.
+*      component
+*         The name of the class structure component that holds the attribute
+*         value.
+*      assign
+*         An expression that evaluates to the value to be assigned to the
+*         component.
+
+*-
+*/
+
+/* Define the macro. */
+#define astMAKE_SET1(class,attribute,type,component,assign) \
+\
+/* Private member function. */ \
+/* ------------------------ */ \
+static void Set##attribute( Ast##class *this, type value, int *status ) { \
+\
+/* Check the inherited error status. */ \
+   if ( !astOK ) return; \
+\
+/* Report an error if the object has been cloned (i.e. has a reference \
+   count that is greater than one). */ \
+   if( astGetRefCount( this ) > 1 ) { \
+      astError( AST__IMMUT, "astSet(%s): The " #attribute "attribute of " \
+                "the supplied %s cannot be changed because the %s has " \
+                "been cloned (programming error).", status, \
+                astGetClass(this), astGetClass(this), astGetClass(this) ); \
+\
+/* Otherwise, store the new value in the structure component. */ \
+   } else { \
+      this->component = (assign); \
+   } \
+} \
+\
+/* External interface. */ \
+/* ------------------- */ \
+void astSet##attribute##_( Ast##class *this, type value, int *status ) { \
+\
+/* Check the inherited error status. */ \
+   if ( !astOK ) return; \
+\
+/* Invoke the required method via the virtual function table. */ \
+   (**astMEMBER(this,class,Set##attribute))( this, value, status ); \
+}
+#endif
+
 #if defined(astCLASS)            /* Protected */
 /*
 *+
@@ -1224,7 +1401,9 @@ int astTest##attribute##_( Ast##class *this, int *status ) { \
 /* Check for equality of floating point values. We cannot compare bad values
    directly because of the danger of floating point exceptions, so bad
    values are dealt with explicitly. */
-#define astEQUAL(aa,bb) (((aa)==AST__BAD)?(((bb)==AST__BAD)?1:0):(((bb)==AST__BAD)?0:(fabs((aa)-(bb))<=1.0E5*astMAX((fabs(aa)+fabs(bb))*DBL_EPSILON,DBL_MIN))))
+#define astEQUALS(aa,bb,tol) (((aa)==AST__BAD)?(((bb)==AST__BAD)?1:0):(((bb)==AST__BAD)?0:(fabs((aa)-(bb))<=(tol)*astMAX((fabs(aa)+fabs(bb))*DBL_EPSILON,DBL_MIN))))
+#define astEQUAL(aa,bb) astEQUALS(aa,bb,1.0E5)
+
 
 /* AST__NULL. */
 /* ---------- */
