@@ -40,20 +40,20 @@ f     The SphMap class does not define any new routines beyond those
 *     Research Councils
 
 *  Licence:
-*     This program is free software; you can redistribute it and/or
-*     modify it under the terms of the GNU General Public Licence as
-*     published by the Free Software Foundation; either version 2 of
-*     the Licence, or (at your option) any later version.
+*     This program is free software: you can redistribute it and/or
+*     modify it under the terms of the GNU Lesser General Public
+*     License as published by the Free Software Foundation, either
+*     version 3 of the License, or (at your option) any later
+*     version.
 *
-*     This program is distributed in the hope that it will be
-*     useful,but WITHOUT ANY WARRANTY; without even the implied
-*     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-*     PURPOSE. See the GNU General Public Licence for more details.
+*     This program is distributed in the hope that it will be useful,
+*     but WITHOUT ANY WARRANTY; without even the implied warranty of
+*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*     GNU Lesser General Public License for more details.
 *
-*     You should have received a copy of the GNU General Public Licence
-*     along with this program; if not, write to the Free Software
-*     Foundation, Inc., 51 Franklin Street,Fifth Floor, Boston, MA
-*     02110-1301, USA
+*     You should have received a copy of the GNU Lesser General
+*     License along with this program.  If not, see
+*     <http://www.gnu.org/licenses/>.
 
 *  Authors:
 *     DSB: David Berry (Starlink)
@@ -79,6 +79,12 @@ f     The SphMap class does not define any new routines beyond those
 *        Modify MapMerge so that it can spot and simplify an
 *        (inverted SphMap,MatrixMap,SphMap) sequence in which the
 *        MatrixMap just magnifies or reflects the radius vector.
+*     25-MAR-2014 (DSB):
+*        Correct 5-NOV-2013 MapMerge change.
+*     28-APR-2016 (DSB):
+*        Avoid modifying the attributes of the existing SphMap in
+*        MapMerge, since it may be in use in other contexts. Modify a
+*        copy instead.
 *class--
 */
 
@@ -104,8 +110,11 @@ f     The SphMap class does not define any new routines beyond those
 #include "channel.h"             /* I/O channels */
 #include "unitmap.h"             /* Unit (identity) Mappings */
 #include "sphmap.h"              /* Interface definition for this class */
-#include "pal.h"              /* SLA transformations */
+#include "pal.h"                 /* SLA transformations */
 #include "wcsmap.h"              /* For the AST__DPIBY2 (etc) constants */
+#include "matrixmap.h"           /* Matrix mappings */
+#include "winmap.h"              /* Shift and scale mappings */
+#include "zoommap.h"             /* Scale mappings */
 
 /* Error code definitions. */
 /* ----------------------- */
@@ -700,6 +709,7 @@ static int MapMerge( AstMapping *this, int where, int series, int *nmap,
 /* Local Variables: */
    AstMapping *new;              /* Pointer to replacement Mapping */
    AstMatrixMap *mm;             /* Pointer to MatrixMap */
+   AstSphMap *sm;                /* The new SphMap */
    AstWinMap *wm;                /* The new WinMap */
    const char *class;            /* Pointer to Mapping class string */
    double absval;                /* Absolute value fo each diagonal element */
@@ -865,10 +875,10 @@ static int MapMerge( AstMapping *this, int where, int series, int *nmap,
          if( diag[ 0 ] < 0.0 ) {
             if( diag[ 1 ] < 0.0 ) {
                wm->a[ 0 ] = AST__DPI;
-               wm->b[ 0 ] = -1.0;
+               wm->b[ 0 ] = 1.0;
             } else {
                wm->a[ 0 ] = AST__DPI;
-               wm->b[ 0 ] = 1.0;
+               wm->b[ 0 ] = -1.0;
             }
 
          } else {
@@ -911,15 +921,19 @@ static int MapMerge( AstMapping *this, int where, int series, int *nmap,
          if( astEQUAL( polarlong, astGetPolarLong( ( *map_list )[ where + 2 ] ) ) ) {
 
 /* All is good, so we can now change the supplied Mappings list. First
-   change the PolarLong value in the first SphMap. */
-            astSetPolarLong( ( *map_list )[ where ], polarlong );
+   get a copy of the first SphMap and change its PolarLong value. We use
+   a copy since the original may be in use in other contexts that could be
+   badly affected by the change. */
+            sm = astCopy( ( *map_list )[ where ] );
+            astSetPolarLong( sm, polarlong );
 
-/* Annul The MatrixMap or ZoomMap. */
+/* Annul the SphMap and the MatrixMap or ZoomMap. */
+            (void) astAnnul( ( *map_list )[ where ] );
             (void) astAnnul( ( *map_list )[ where + 1 ] );
 
-/* Move the first SphMap to the slot left vacant by the annulled
+/* Store the modified SphMap in the slot left vacant by the annulled
    MatrixMap or ZoomMap. */
-            ( *map_list )[ where + 1 ] = ( *map_list )[ where ];
+            ( *map_list )[ where + 1 ] = (AstMapping *) sm;
             ( *invert_list )[ where + 1 ] = ( *invert_list )[ where ];
 
 /* Store the new WinMap in the place of the SphMap. */
@@ -1353,15 +1367,22 @@ f     therefore have the same coordinate values.
 *     the coordinates transformed by a SphMap. The lengths of the
 *     input 3-dimensional Cartesian vectors supplied are always
 *     ignored, even if UnitRadius is non-zero.
+*     - The value of this attribute may changed only if the SphMap
+*     has no more than one reference. That is, an error is reported if the
+*     SphMap has been cloned, either by including it within another object
+*     such as a CmpMap or FrameSet or by calling the
+c     astClone
+f     AST_CLONE
+*     function.
 
 *  Applicability:
 *     SphMap
 *        All SphMaps have this attribute.
 *att--
 */
-astMAKE_CLEAR(SphMap,UnitRadius,unitradius,-1)
+astMAKE_CLEAR1(SphMap,UnitRadius,unitradius,-1)
 astMAKE_GET(SphMap,UnitRadius,int,0,(this->unitradius == -1 ? 0 : this->unitradius))
-astMAKE_SET(SphMap,UnitRadius,int,unitradius,( value ? 1 : 0 ))
+astMAKE_SET1(SphMap,UnitRadius,int,unitradius,( value ? 1 : 0 ))
 astMAKE_TEST(SphMap,UnitRadius,( this->unitradius != -1 ))
 
 /* PolarLong */
@@ -1385,15 +1406,23 @@ astMAKE_TEST(SphMap,UnitRadius,( this->unitradius != -1 ))
 *     returned when a Cartesian position corresponding to either the north
 *     or south pole is transformed into spherical coordinates. The
 *     default value is zero.
+*
+*     Note, the value of this attribute may changed only if the SphMap
+*     has no more than one reference. That is, an error is reported if the
+*     SphMap has been cloned, either by including it within another object
+*     such as a CmpMap or FrameSet or by calling the
+c     astClone
+f     AST_CLONE
+*     function.
 
 *  Applicability:
 *     SphMap
 *        All SphMaps have this attribute.
 *att--
 */
-astMAKE_CLEAR(SphMap,PolarLong,polarlong,AST__BAD)
+astMAKE_CLEAR1(SphMap,PolarLong,polarlong,AST__BAD)
 astMAKE_GET(SphMap,PolarLong,double,0.0,(this->polarlong == AST__BAD ? 0.0 : this->polarlong))
-astMAKE_SET(SphMap,PolarLong,double,polarlong,value)
+astMAKE_SET1(SphMap,PolarLong,double,polarlong,value)
 astMAKE_TEST(SphMap,PolarLong,( this->polarlong != AST__BAD ))
 
 /* Copy constructor. */
